# -*- coding: utf-8 -*-
"""
app.helpers
~~~~~~~~~~~

This module implements helpers functions for the models.

:copyright: (c) 2017 European Spallation Source ERIC
:license: BSD 2-Clause, see LICENSE for more details.

"""
import string
from flask import current_app
from flask_wtf import FlaskForm
from . import models


class CSEntryForm(FlaskForm):

    def __init__(self, formdata=None, obj=None, **kwargs):
        # Store obj for Unique validator to check if the unique object
        # is identical to the one being edited
        self._obj = obj
        # formdata is often given as first argument (not keyword argument)
        # It's initialized as a singleton in flask-wtf so we should only pass
        # it if not None
        if formdata is None:
            super().__init__(obj=obj, **kwargs)
        else:
            super().__init__(formdata=formdata, obj=obj, **kwargs)


def temporary_ics_ids():
    """Generator that returns the full list of temporary ICS ids"""
    return (f'{current_app.config["TEMPORARY_ICS_ID"]}{letter}{number:0=3d}'
            for letter in string.ascii_uppercase
            for number in range(0, 1000))


def used_temporary_ics_ids():
    """Generator that returns the list of temporary ICS ids used"""
    temporary_items = models.Item.query.filter(
        models.Item.ics_id.startswith(
            current_app.config['TEMPORARY_ICS_ID'])).all()
    return (item.ics_id for item in temporary_items)


def get_temporary_ics_id():
    """Return a temporary ICS id that is available"""
    for ics_id in temporary_ics_ids():
        if ics_id not in used_temporary_ics_ids():
            return ics_id
    else:
        raise ValueError('No temporary ICS id available')
